/*

    Copyright (C) 2020 willipink.eu
    Author Moritz Münch moritzmuench@mailbox.org

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program. If not, see <http://www.gnu.org/licenses/>. */


import { Cookie } from './modules/cookie.mjs';

let settings = {
    found:        true,
    found_by:     '',
    found_on:     '',
    circulation:  0,
    buy_only:     false,
    checked:      false,
    ec:           false,
    exists:       true
}

//
// Functions
//

// return a datetime YYYY.MM.DD
function get_datetime() {
    let datetime = new Date();
    return ('0' +  datetime.getDate()).slice(-2)       + '.' +
           ('0' + (datetime.getMonth() + 1)).slice(-2) + '.' +
                   datetime.getFullYear();
}


// manage user login
function render_login(input_name, input_color) {

    let color = decodeURIComponent(input_color);
    let name = decodeURIComponent(input_name);
    new Cookie('coinc-name', name)

    render_login_color(color);
    let option = $('nav.navbar option[value=' + name + ']');
    if (option.length == 0) {
        $('nav.navbar select').append($('<option>', {
            value: name, color: color, html: name
        }));
    }

    $('nav.navbar option[value=' + name + ']').attr('selected', 'selected');
}


// manage login colors
function render_login_color(color) {

    new Cookie('coinc-color', color);
    if ($('nav.navbar').css('display') == 'block') {
        $('nav.navbar').prop('style', 'background-color:' + color + ' !important; display: block;');
    } else {
        $('nav.navbar').attr('style', 'background-color:' + color + ' !important;');
    }
}


// show/update #found_by and #found_on
function render_found() {
    let checked = $('#found').prop('checked');
    if (checked === true) {
        $('#found_by option[selected=selected]').html(settings['found_by']);
        let found_on = ''
        if (settings['found_on']) {
            found_on = settings['found_on'];
        } else {
            found_on = get_datetime();
        }
        $('#found_on').val(found_on);
    } else {
        save_settings();
        $('#found_by option[selected=selected]').html('');
        $('#found_on').val('');
    }

    $('#found_by, #found_on').each(function() {
        if (checked === true) {
            $(this).prop('disabled', '');
        } else {
            $(this).prop('disabled', 'disabled');
        }
    });
}


// update table cell according to settings
function render_response(td, set) {
    let td_class = '';
    let div_class = '';

    if (!set['exists']) {
        td_class = 'noexist';
    } else if (set['buy_only']) {
        td_class = 'buy_only';
    } else if (set['circulation'] && set['circulation'] < 1500000 && set['circulation'] != 0) {
        td_class = 'rare';
    } else if ([10, 20, 50].includes(Number($(td).attr('value')))) {
        td_class = 'brass';
    } else if (set['found'] && set['found_by']) {
        td_class = 'found';
    }

    if (set['ec']) {
        td_class += ' ec';
    } else if (set['checked']) {
        td_class += ' checked';
    }

    if (set['found'] && set['found_by'] && set['exists']) {
        div_class = set['found_by'];
        $(td).find('div.coin span').html('').html('x');
    }
    if (!set['exists'] || !set['found']) {
        $(td).find('div.coin span').html('');
    }
    $(td).removeClass();
    $(td).addClass('coin ' + td_class);
    $(td).find('div[type=overlay]').removeClass();
    $(td).find('div[type=overlay]').addClass('found ' + div_class);
}


// save/update global settings
function save_settings() {

    let checkboxes = $('#found, #buy_only, #checked, #ec, #exists').map(function() {
        return { [$(this).prop('id')]: $(this).prop('checked') }
    }).get();
    for (let box of checkboxes) {
        for (let item in box) { settings[item] = box[item]; }
    }
    settings['exists'] = !settings['exists']

    let found_by = $('#found_by option[selected=selected]').html();
    if (found_by && settings['exists'] && settings['found']) {
        settings['found_by'] = found_by;
    }

    let found_on = $('#found_on').val();
    let pattern = /(\d{2})\.(\d{2})\.(\d{4})/;
    if (!isNaN(found_on) && isNaN(Date.parse(found_on.replace(pattern, '$3-$2-$1')))) {;
        settings['found_on'] = get_datetime();
    } else {
        settings['found_on'] = found_on;
    }

    let circulation = $('#circulation').val();
    if (isNaN(parseInt(circulation)) || circulation < 0 || circulation > 1000000000) {
        settings['circulation'] = 0;
    } else {
        settings['circulation'] = circulation;
    }
}


$(document).ready(function() {

    //
    // Buttons
    //

    // button #begin_edit
    $('#begin_edit').click(function() {
        $('nav.navbar').fadeIn('fast');
        $('#begin_edit').fadeOut('fast');

        // add coin
        $('td.coin').each(function() {
            $(this).click(function() {
                save_settings();

                let csrf_token = $('input[name=csrfmiddlewaretoken]').val();
                let value = $(this).attr('value');
                let year = Number($(this).parent('tr').attr('name'));
// TODO fix this before the end of 2098
                year += (year == 99) ? 1900 : 2000;
                let country = $(this).parents('table').attr('name');
                let stamp = $(this).parent('tr').attr('stamp');
                let name = ''
                let found_by = (settings['found'] === true) ? settings['found_by'] : ''
                let found_on = (settings['found'] === true) ? settings['found_on'] : ''
                let circulation = (settings['circulation'] == 'Auflage') ? 0 : settings['circulation'];

                if (settings['exists'] === false) { circulation = 0; }

                let data = {
                    csrfmiddlewaretoken: csrf_token,
                    value: value,
                    year: year,
                    country: country,
                    stamp: stamp,
                    name: name,
                    found_by: found_by,
                    found_on: found_on,
                    circulation: circulation,
                    buy_only: settings['buy_only'],
                    checked: settings['checked'],
                    ec: settings['ec'],
                    exists: settings['exists']
                }

                $.ajax({
                    type: 'POST',
                    url: 'add/coin',
                    data: data,
                    dataType: 'json',
                    td: this,
                    success: function(response) {
                        if (response.status === 0) {
                            render_response(this.td, settings);
                        } else {
                            alert(response.message);
                        }
                    }
                });
            });
        });
    });

    // button #finish_edit
    $('#finish_edit').click(function() {
        $('nav.navbar').fadeOut('fast');
        $('#begin_edit').fadeIn('fast');
    });

    // button #advanced_options
    $('#show_advanced_options').click(function() {
        $('#advanced_options').slideToggle();
    });



    //
    // Misc
    //

    // checkbox #found
    $('#found').change(function() {
        render_found();
    });

  // checkbox #exists
  $('#exists').change(function() {

    if (this.checked === true) {

      // save settings
      save_settings();

      // remove values
      $('#found_by, #found_on, #circulation').each(function() {
          $(this).val('');
      });

      // disable settings
      $('#found, #buy_only, #checked, #ec').each(function() {
          $(this).prop('checked', false);
      });
      $('#found, #found_by, #found_on, #circulation, #buy_only, #checked, #ec').each(function() {
          $(this).prop('disabled', 'disabled');
      });

    // checkbox #exists not checked
    } else {

      // restore values
      $('#found_by').val(settings['found_by']);
      $('#found_on').val(settings['found_on']);
      $('#circulation').val(settings['circulation']);

      // restore settings
      $('#found_now, #found, #buy_only, #checked, #ec').each(function() {
        let setting = $(this).prop('id');
        $(this).prop('checked', settings[setting]);
      });

      // enable settings
      $('#found, #found_by, #found_on, #circulation, #buy_only, #checked, #ec').each(function() {
        $(this).prop('disabled', '');
      });

      // do post processing
      render_found();

    }
  });

  // select username on change
  $('nav.navbar select').on('change', function() {
    let color = $('nav.navbar option[value=' + this.value + ']').attr('color');
    $('nav.navbar option[selected=selected]').removeAttr('selected');
    render_login(this.value, color);
  });

// datepicker TODO make it nicer
  $.datepicker.setDefaults( $.datepicker.regional['de'] );
  $('input[name=found_on]').datepicker({
    defaultDate: new Date(),
    dateFormat: 'dd.mm.yy'
  });




  //
  // Modals
  //

  // hide modal add user
  function hide_modal() {
      $('#modal_add_user').fadeOut('fast');
  }

  // modal #add_user
  $('#show_add_user').click(function() {
    $('#modal_add_user').fadeIn('fast');

    $(document).on('click', function(event) {
      if ($('#modal_add_user').is(':visible')) {
        let target = $(event.target);
        if (target.attr('id') === 'modal_add_user') {
          hide_modal();
        }
      }
    });

    // detect ESC keypress
    $(document).keypress(function(event) {
      if (event.keyCode == 27) { // ESC is pressed
        hide_modal();
        $(document).off('keypress');
      }
    });

    // button #close_modal_add_user
    $('#close_modal_add_user').click(function() {
      hide_modal();
    });


    // button #add_user
    $('#add_user').click(function() {

      let name = encodeURIComponent($('#text_user').val());
      let color = encodeURIComponent($('#text_color').val());

      $.ajax({
        type: 'GET',
        dataType: 'json',
        url: 'add/user/' + name + '/color/' + color,
        success: function(response) {

          if (response.status === 0) {
            $('#response').fadeOut('fast');
            hide_modal();
            render_login(name, color);

          } else {
            $('#response').html(response.message);
            $('#response').css('color', 'red');
          }
        }
      });
    });

  });


  //
  // Main routine
  //

  let cookie = new Cookie();
  let name = cookie.get('coinc-name');
  let color = cookie.get('coinc-color');

  if (name != '') {
    render_login(name, color);

  } else {
    name = $('#found_by option:first-child').html();
    color = $('#found_by option:first-child').val();
    render_login(name, color);
  }

  save_settings()
  render_found();

});
